// Plasma printer
// Details on: https://homofaciens.de/technics-machines-plasma-printer_en.htm

$fn=100;

M3_hole = 3.4;
M3_nut_e = 6.4;
M4_hole = 4.4;
M4_nut_e = 7.66+0.5;
M5_nut_e = 8.79+0.5;
M6_nut_e = 11.05+0.5;

// Ball bearing dimensions
bb_di = 6;
bb_do = 19;
bb_h = 6;
bb_m_d = 25;

// Dimensions paper feed
pf_d = 6;
pf_l = 75;
pf_h = 40;

// Dimensions elektrode frame
ef_d = 1;
ef_w = 48;
ef_l = pf_l*2+40;
ef_h = pf_h;
ef_wall = 2;
ef_gap = 15;
ef_m_d = 30;

// Dimensions top roll mount for paper feed
trm_w = 30;
trm_h = 10;
trm_wall = 2;
trm_m_d = 33;

Assembled = 1;

if(Assembled){
  translate([0, 0, 0])rotate([0, 0, 0]){
    paper_feed_clamp();
  }
  translate([pf_l+20, 0, 0])rotate([0, 0, 180]){
    paper_feed_clamp();
  }
  translate([pf_l+20, 0, 33])rotate([0, 270, 0]){
    bb_mount();
  }
  translate([12, 0, 50])rotate([0, 90, 0]){
    top_roll_mount(-7);
  }
  translate([8+pf_l, 0, 50])rotate([0, 90, 180]){
    top_roll_mount(7);
  }
  translate([-40, 0, 0])rotate([0, 0, 0]){
    stepper_mount_feed();
  }
  translate([100, 60, 0])rotate([0, 0, 270]){
    stepper_mount_x();
  }
  translate([100, 35, 22.5])rotate([90, 0, 0]){
    gear(teeth = 15, h = 5, shaft = 5);
  }
  
  translate([-30, 0, pf_h])rotate([0, 90, 0]){
    motor_coupler();
  }
  translate([15+60*0, 25.5, pf_h+pf_d/2])rotate([90, 0, 180]){
    elektrode_frame();
  }
  translate([47.5, 27.5, pf_h+pf_d/2])rotate([0, 180, 0]){
    paper_guide(1);
  }
  translate([47.5, 27.5, pf_h+pf_d/2+1])rotate([180, 180, 180]){
    paper_guide(2);
  }

}
else{
//  translate([0, 0, 12.5])rotate([0, 90, 0])paper_feed_clamp();
//  translate([0, 0, 12.5])rotate([0, 90, 0])stepper_mount_feed();
//  translate([0, 0, 10])rotate([0, 90, 0])stepper_mount_x();
//  top_roll_mount(7);
//  translate([12, 14, 0])top_roll_mount(-7);
//  motor_coupler();
//  elektrode_frame();
//  bb_mount();
  gear(teeth = 10, h = 5, shaft = 5);
//  paper_guide(1);
//  paper_guide(2);
//  M3_knob();
//  electrode_mount();
//  wire_mount();
}

module wire_mount(){
  wm_l = 40;
  wm_w = 10;
  wm_h = 3;
  wm_screw_d = 20;
  wm_w_d = 27;
  
  difference(){
    roundedRect([wm_w, wm_l, wm_h], wm_w/2);
    // Mount holes
    translate([0, wm_screw_d/2, wm_h/2]){
      cylinder(d=M3_hole, h=wm_h*1.1, center=true);
    }
    translate([0, -wm_screw_d/2, wm_h/2]){
      cylinder(d=M3_hole, h=wm_h*1.1, center=true);
    }
    // Wire grove
    translate([0, wm_w_d/2, wm_h]){
      rotate([0, 90, 0]){
        cylinder(d=1, h=wm_w*1.1, center=true);
      }
    }
    translate([0, -wm_w_d/2, wm_h]){
      rotate([0, 90, 0]){
        cylinder(d=1, h=wm_w*1.1, center=true);
      }
    }
  }
  
  
}

module electrode_mount(){
  em_w = 10;
  em_l = 16;
  em_h = 5;
  em_m_d = 13; 
  
  translate([0, 0, em_h/2]){
    difference(){
      cube(size=[em_l, em_w, em_h], center=true);
      translate([0, 0, em_h/2]){
        rotate([90, 0, 0]){
          cylinder(d=1, h=em_w*1.1, center=true);
        }
      }
      translate([0, 3, em_h/2]){
        rotate([0, 90, 0]){
          cylinder(d=1, h=em_w*1.1, center=true);
        }
      }
      translate([0, 3, em_h/2]){
        rotate([0, 0, 0]){
          cylinder(d=3, h=1, center=true);
        }
      }
      hull(){
        translate([em_m_d/2, 0, 0]){
          cylinder(d=M3_hole, h=em_h*1.1, center=true);
        }
        translate([em_l/2, 0, 0]){
          cylinder(d=M3_hole, h=em_h*1.1, center=true);
        }
      }
      hull(){
        translate([-em_m_d/2, 0, 0]){
          cylinder(d=M3_hole, h=em_h*1.1, center=true);
        }
        translate([-em_l/2, 0, 0]){
          cylinder(d=M3_hole, h=em_h*1.1, center=true);
        }
      }
      translate([em_l/2, 0, em_h/2]){
        cube(size=[10, em_w*1.1, 6], center=true);
      }
      translate([-em_l/2, 0, em_h/2]){
        cube(size=[10, em_w*1.1, 4], center=true);
      }
    }
  }
  
}

module M3_knob(){
  m3k_h = 15;
  m3k_h_thread = 3;
  m3k_h2 = 3;
  m3k_d = 8;
  m3k_d2 = 17;
  m3k_hole = 3;
  m3k_hole2 = 4;

  // Knob  
  translate([0, 0, m3k_h2/2]){
    difference(){
      cylinder(d=m3k_d2, h=m3k_h2, center=true);
      cylinder(d=m3k_hole2, h=m3k_h*1.1, center=true);
    }
  }
  translate([0, 0, m3k_h/2]){
    difference(){
      cylinder(d=m3k_d, h=m3k_h, center=true);
      translate([0, 0, -m3k_h_thread]){
        cylinder(d1=m3k_hole2, d2=m3k_hole, h=m3k_h, center=true);
      }
      cylinder(d=m3k_hole, h=m3k_h*1.1, center=true);
    }
  }
  
}

module paper_guide(type = 2){
  pg_l = pf_l-4;
  pg_w = 30;
  pg_h = 20;
  pg_wall = 2;
  pg_wall2 = 4;
  pg_gap_l = pg_l-5;
  pg_gap_w = 10;
  pg_enf_w = 10;
  
  difference(){
    union(){
      // Base
      if(type == 1){
        roundedRect([pg_l, pg_w, pg_wall2], 5, [0,0,1,1]);
      }
      else{
        roundedRect([pg_l, pg_w, pg_wall], 5, [0,0,1,1]);
      }
      // Rear wall
      translate([0, -pg_w/2, pg_h/2]){
        rotate([90, 0, 0]){
          roundedRect([pg_l, pg_h, pg_wall], 5, [0,0,0,0]);
        }
      }
      // Side walls
      for(i=[0:1:1])mirror([i,0,0])
      translate([(pg_l-pg_wall)/2, (pg_enf_w-pg_w)/2, pg_h/2]){
        difference(){
          cube(size=[pg_wall, pg_enf_w, pg_h], center=true);
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pg_wall2*1.1, center=true);
          }
        }
      }
    }
    // Center cut
    translate([0, (pg_w-pg_gap_w)/2-5, -0.1]){
      roundedRect([pg_gap_l, pg_gap_w, pg_wall2*1.1], 2);
    }
  }
  
}
module elektrode_frame(){
  h_shift = -3;
  difference(){
    union(){
      translate([0, h_shift, 0]){
        roundedRect([ef_l, ef_w, ef_wall], 5);
      }
      translate([93, h_shift-ef_w/2-0, 0]){
        rotate([0, 0, 90]){
          gear_rod(teeth = 62, gr_w = 8, gr_h = 3);
        }
      }
    }
    translate([0, 0, -0.1]){
      roundedRect([pf_l*2, ef_gap, ef_wall*1.1], 5);
    }
    // mount holes
    for(i=[0:1:1])mirror([0,i,0]){
      translate([ef_l/2-5, 10, 0]){
        cylinder(d=M3_hole, h=ef_wall*2.1, center=true);
      }
      translate([5-ef_l/2, 10, 0]){
        cylinder(d=M3_hole, h=ef_wall*2.1, center=true);
      }
    }
    // Guiding slits
    for(i=[0:1:1])mirror([0,i,0]){
      translate([0, ef_m_d/2, 0]){
        cylinder(d=M3_hole, h=ef_wall*2.1, center=true);
      }
      translate([pf_l/2+5, ef_m_d/2, -0.1]){
        roundedRect([pf_l, M3_hole+0.5, ef_wall*1.1], M3_hole/2);
      }
      translate([-pf_l/2-5, ef_m_d/2, -0.1]){
        roundedRect([pf_l, M3_hole+0.5, ef_wall*1.1], M3_hole/2);
      }
    }
  }
  
}

module motor_coupler(){
  mc_do = 20;
  mc_di = 5.1;
  mc_l = 20;
  
  translate([0, 0, mc_l/2]){
    difference(){
      cylinder(d=mc_do, h=mc_l, center=true);
      cylinder(d=mc_di, h=mc_l*1.1, center=true);
      translate([0, 0, mc_l/2]){
        cylinder(d=M6_nut_e, h=10, center=true, $fn=6);
      }
      translate([0, 2+mc_di/2+1, -mc_l/2]){
        cube(size=[5.5+0.5, 4 ,mc_l-2], center=true);
      }
      
      translate([0, 0, -mc_l/4]){
        rotate([90, 0, 0]){
          cylinder(d=M3_hole, h=mc_l, center=true);
        }
      }
    }
  }
}

module top_roll_mount(shift = -7){
  difference(){
    union(){
      translate([0, shift, 0]){
        roundedRect([trm_h, trm_w, trm_wall], 4);
      }
      translate([(trm_h-pf_d-trm_wall)/2, 0, 2.5]){
        difference(){
          cylinder(d=pf_d+2*trm_wall, h=5, center=true);
          cylinder(d=pf_d, h=10, center=true);
          translate([pf_d/2, 0, 0]){
            cube(size=[pf_d,pf_d,10], center=true);
          }
        }
      }
    }
    translate([0, trm_m_d/2, 0]){
      cylinder(d=M3_hole, h=trm_wall*2.1, center=true);
    }
    translate([0, -trm_m_d/2, 0]){
      cylinder(d=M3_hole, h=trm_wall*2.1, center=true);
    }
  }
  
}

module bb_mount(){
  bbm_w = 22;
  bbm_h = 38;
  bbm_thick = 8.5;
  bbm_wall = 2;
  
  difference(){
    roundedRect([bbm_h,bbm_w,bbm_thick], bbm_w/2);
    translate([(bbm_h-bb_do)/2-bbm_wall, 0, bbm_thick/2+bbm_wall]){
      cylinder(d=bb_do+0.2, h=bbm_thick, center=true);
    }
    translate([(bbm_h-bb_do)/2-bbm_wall, 0, 0]){
      cylinder(d=12, h=bbm_thick*2.1, center=true);
      translate([-bb_m_d/2, 0, 0]){
        cylinder(d=M3_hole, h=bbm_thick*2.1, center=true);
      }
      translate([-bb_m_d/2-10, 0, 0]){
        cylinder(d=M3_hole, h=bbm_thick*2.1, center=true);
      }
    }
  }
  
}

module paper_feed_clamp(pfc_hole = 12){
  // Paper feed clamp dimensions
  pfc_w = 50;
  pfc_l = 20;
  pfc_h = 70;
  pfc_wall = 2;
  pfc_m_d = 20;
  
  difference(){
    union(){
      //Base plate
      roundedRect([pfc_l, pfc_w, pfc_wall], 5, [1,0,1,0]);
      // Back wall
      translate([pfc_l/2, 0, pfc_h/2]){
        rotate([0, 90, 0]){
          roundedRect([pfc_h, pfc_w, pfc_wall], 5, [1,0,1,0]);
        }
      }
      // Enforcement
      for(i=[0:1:1])mirror([0,i,0])
      hull(){
        translate([(pfc_l+pfc_wall)/2, (pfc_w-pfc_wall)/2, pfc_wall/2]){
          cube(size=[pfc_wall, pfc_wall, pfc_wall], center=true);
        }
        translate([(pfc_wall-pfc_l)/2+5, (pfc_w-pfc_wall)/2, pfc_wall/2]){
          cube(size=[pfc_wall, pfc_wall, pfc_wall], center=true);
        }
        translate([(pfc_l+pfc_wall)/2, (pfc_w-pfc_wall)/2, pfc_h-5]){
          cube(size=[pfc_wall, pfc_wall, pfc_wall], center=true);
        }
        translate([(pfc_l-12)/2, (pfc_w-pfc_wall)/2, pfc_h-5-7/2]){
          rotate([90, 0, 0]){
            cylinder(d=7, h=pfc_wall, center=true);
          }
        }
      }
    }
    // Ball bearing mount
    translate([pfc_l/2, 0, pf_h-bb_m_d/2]){
      rotate([0, 90, 0]){
        cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
      }
    }
    translate([pfc_l/2, 0, pf_h-bb_m_d/2-10]){
      rotate([0, 90, 0]){
        cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
      }
    }
    // Paper feed cut
    translate([pfc_l/2, 0, pf_h]){
      rotate([0, 90, 0]){
        cylinder(d=pfc_hole, h=pfc_wall*2.1, center=true);
      }
    }
    // Bottom mount holes
    translate([0, pfc_m_d/2, 0]){
      cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
    }
    translate([0, -pfc_m_d/2, 0]){
      cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
    }
    // X axis (electrodes frame) mount holes    
    translate([5, 0, pf_h+(pf_d-ef_m_d)/2]){
      rotate([90, 0, 0]){
        cylinder(d=M3_hole, h=pfc_w*2.1, center=true);
      }
    }
    translate([5, 0, pf_h+(ef_m_d+pf_d)/2]){
      rotate([90, 0, 0]){
        cylinder(d=M3_hole, h=pfc_w*2.1, center=true);
      }
    }
    
    
    // Top paper feed roll mount
    for(i=[0:1:1])mirror([0,i,0]){
      hull(){
        translate([pfc_l/2, trm_m_d/2, pf_h+7]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
          }
        }
        translate([pfc_l/2, trm_m_d/2, pf_h+20]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
          }
        }
      }
      // Elektrode mount
      hull(){
        translate([pfc_l/2, 16.5, pf_h]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
          }
        }
        translate([pfc_l/2, 16.5, pf_h-15]){
          rotate([0, 90, 0]){
            cylinder(d=M3_hole, h=pfc_wall*2.1, center=true);
          }
        }
      }
    }
    
  }
  
}


module stepper_mount_x(){
  // Paper feed clamp dimensions
  sm_w = 50;
  sm_w2 = 70;
  sm_l = 20;
  sm_h = 45;
  sm_wall = 2;
  sm_m_d = 60;
  stepper_m_d = 31;
  
  difference(){
    union(){
      //Base plate
      roundedRect([sm_l, sm_w2, sm_wall], 5, [1,1,1,1]);
      // Back wall
      translate([sm_l/2-sm_wall, 0, sm_h/2]){
        rotate([0, 90, 0]){
          roundedRect([sm_h, sm_w, sm_wall], 5, [1,0,1,0]);
        }
      }
      for(i=[0:1:1])mirror([0,i,0])
      hull(){
        translate([(sm_l-sm_wall)/2, (sm_w-sm_wall)/2, sm_wall/2]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
        translate([(sm_wall-sm_l)/2+1, (sm_w-sm_wall)/2, sm_wall/2]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
        translate([(sm_l-sm_wall)/2, (sm_w-sm_wall)/2, sm_h-5]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
      }
    }
    // Motor axis cut
    translate([sm_l/2, 0, sm_h/2]){
      rotate([0, 90, 0]){
        cylinder(d=23, h=sm_wall*2.1, center=true);
      }
    }
    // Bottom mount holes
    translate([0, sm_m_d/2, 0]){
      cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
    }
    translate([0, -sm_m_d/2, 0]){
      cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
    }
    // Stepper motor mount
    for(i=[0:1:1])mirror([0,i,0]){
      translate([sm_l/2, stepper_m_d/2, (sm_h+stepper_m_d)/2]){
        rotate([0, 90, 0]){
          cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
        }
      }
      translate([sm_l/2, stepper_m_d/2, (sm_h-stepper_m_d)/2]){
        rotate([0, 90, 0]){
          cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
        }
      }
    }
    
  }
  
}


module stepper_mount_feed(){
  // Paper feed clamp dimensions
  sm_w = 50;
  sm_l = 20;
  sm_h = 60;
  sm_wall = 2;
  sm_m_d = 20;
  stepper_m_d = 31;
  
  difference(){
    union(){
      //Base plate
      roundedRect([sm_l, sm_w, sm_wall], 5, [1,0,1,0]);
      // Back wall
      translate([sm_l/2, 0, sm_h/2]){
        rotate([0, 90, 0]){
          roundedRect([sm_h, sm_w, sm_wall], 5, [1,0,1,0]);
        }
      }
      for(i=[0:1:1])mirror([0,i,0])
      hull(){
        translate([(sm_l+sm_wall)/2, (sm_w-sm_wall)/2, sm_wall/2]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
        translate([(sm_wall-sm_l)/2+5, (sm_w-sm_wall)/2, sm_wall/2]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
        translate([(sm_l+sm_wall)/2, (sm_w-sm_wall)/2, sm_h-5]){
          cube(size=[sm_wall, sm_wall, sm_wall], center=true);
        }
      }
    }
    // Motor axis cut
    translate([sm_l/2, 0, pf_h]){
      rotate([0, 90, 0]){
        cylinder(d=23, h=sm_wall*2.1, center=true);
      }
    }
    // Bottom mount holes
    translate([0, sm_m_d/2, 0]){
      cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
    }
    translate([0, -sm_m_d/2, 0]){
      cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
    }
    // Stepper motor mount
    for(i=[0:1:1])mirror([0,i,0]){
      translate([sm_l/2, stepper_m_d/2, pf_h+stepper_m_d/2]){
        rotate([0, 90, 0]){
          cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
        }
      }
      translate([sm_l/2, stepper_m_d/2, pf_h-stepper_m_d/2]){
        rotate([0, 90, 0]){
          cylinder(d=M3_hole, h=sm_wall*2.1, center=true);
        }
      }
    }
    
  }
  
}

module gear(teeth = 10, h = 5, shaft = M3_hole){
  mod = 3; 
  alpha = 180/teeth;
  r1 = mod*teeth/(2*3.14159);
  r=(mod/2)/tan(alpha);
  edge = 1;
  
  difference(){
    union(){
      x1=r;
      y1=mod/2;
      x2=r+mod;
      y2=0;
      x3=r;
      y3=-mod/2;
      x4=0;
      y4=0;
      translate([0, 0, edge]){
        linear_extrude(height=h-edge){
          for(i=[0:1:teeth]){
            rotate([0, 0, i*360/teeth]){
              polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
            }
          }
        }
      }
      translate([0, 0, edge]){
        mirror([0, 0, 1]){
          linear_extrude(height=edge, scale=[0.97, 0.97]){
            for(i=[0:1:teeth]){
              rotate([0, 0, i*360/teeth]){
                polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
              }
            }
          }
        }
      }
    }
    translate([0, 0, h/2]){
      cylinder(d=shaft, h=h*1.1, center=true);
    }
    if(shaft==-1){
      translate([0, 0, h/2]){
        difference(){
          cylinder(d=motor_shaft, h=h*1.1, center=true);
          translate([0, motor_shaft_shift, 0]){
            cube(size=[motor_shaft, motor_shaft, 16], center=true);
          }
        }
      }
    }
  }
  
}

module gear_rod(teeth = 10, gr_w = 7, gr_h = 3){
  mod = 3; 
  alpha = 180/teeth;
  r1 = mod*teeth/(2*3.14159);
  r=(mod/2)/tan(alpha);
  
  difference(){
    union(){
      x1=gr_h;
      y1=mod/2;
      x2=gr_h+mod;
      y2=0;
      x3=gr_h;
      y3=-mod/2;
      x4=0;
      y4=-mod/2;
      x5=0;
      y5=mod/2;
      translate([0, 0, 0]){
        linear_extrude(height=gr_w){
          for(i=[0:1:teeth]){
            translate([0, i*3, 0]){
              polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5]]);
            }
          }
        }
      }
    }
  }
  
}

module roundedRect(size, radius, edges=[1, 1, 1, 1]){
  x = size[0];
  y = size[1];
  z = size[2];

  linear_extrude(height=z){
    hull(){
        // place 4 circles in the corners, with the given radius
        if(edges[0]){
          translate([(-x/2)+(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
        if(edges[1]){
          translate([(x/2)-(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[2]){
          translate([(-x/2)+(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[3]){
          translate([(x/2)-(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
    }
  }
}