// Control for HP 6602 ink jet print head

#define LED_1        13

#define PLASMA        9

#define DIR_X         6
#define STEP_X        7
#define DIR_Y         4
#define STEP_Y        5

#define STEPPER_EN    8

#define STEP_PAUSE       1000
#define STEP_PAUSE_FAST   150
#define COMMAND_PAUSE       5
#define DOT_PAUSE_LONG      100
#define DOT_PAUSE_SHORT      20

int i;
char comByte = 0;
char dataMode = 0;
long dotWidth = 36;
long lineHeight = 49;
long stepsDone = 0;



// Initialize communication through USB
void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(100);
  }
}


void motorStep(long stepWidth, int dirPin, int stepPin, long stepPause, int sleepAfterStep){
  digitalWrite(STEPPER_EN, LOW);// Signal is NOT ENABLE!
  if(stepWidth < 0){
    digitalWrite(dirPin, HIGH);
    stepWidth = -stepWidth;
  }
  else{
    digitalWrite(dirPin, LOW);
  }
  for(long i=0; i<stepWidth; i++){
    digitalWrite(stepPin, LOW);
    delayMicroseconds(COMMAND_PAUSE);
    digitalWrite(stepPin, HIGH);
    delayMicroseconds(stepPause);
  }
  if(sleepAfterStep == 1){
    digitalWrite(STEPPER_EN, HIGH);
  }
}

void makeDot(long dotPause){
  digitalWrite(PLASMA, HIGH);
  delay(dotPause);
  digitalWrite(PLASMA, LOW);
}

// the setup routine runs once when you press reset:
void setup(){
  dataMode = 0;
  stepsDone = 0;

  pinMode(LED_1, OUTPUT);

  pinMode(DIR_X, OUTPUT);
  pinMode(STEP_X, OUTPUT);
  pinMode(DIR_Y, OUTPUT);
  pinMode(STEP_Y, OUTPUT);

  pinMode(STEPPER_EN, OUTPUT);
  pinMode(PLASMA, OUTPUT);

//  digitalWrite(PLASMA, HIGH);
//  delay(2000);
//  digitalWrite(PLASMA, LOW);

//  makeDot(200);

  digitalWrite(STEPPER_EN, HIGH);

/*
  motorStep(100, DIR_X, STEP_X, STEP_PAUSE);
  motorStep(-100, DIR_X, STEP_X, STEP_PAUSE);
  motorStep(100, DIR_Y, STEP_Y, STEP_PAUSE);
  motorStep(-100, DIR_Y, STEP_Y, STEP_PAUSE);

  for (int j = 0; j < 20; j++){
    for(i = 0; i < 20; i++){
      makeDot(55 + i * 5);
      motorStep(300, DIR_X, STEP_X, STEP_PAUSE, 0);
    }
    motorStep(1000, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 1);
    motorStep(-300 * 20, DIR_X, STEP_X, STEP_PAUSE_FAST, 1);        
  }
*/
  
  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();
}

// the loop routine runs over and over again forever:
void loop() {

  comByte = 0;
  if (Serial.available() > 0){//if valid, read from serial:
    //get incoming byte:
    comByte = Serial.read();
    Serial.print('R');   // send 'R' to initiate next data from computer

    if(dataMode == 0){

      if(comByte == 'd'){
        dataMode = 1;
      }

      if(comByte == 'r'){// Return line
        motorStep(-stepsDone, DIR_X, STEP_X, STEP_PAUSE_FAST, 0);
        stepsDone = 0;
      }
      if(comByte == 'n'){// New line
        motorStep(lineHeight, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 0);
        stepsDone = 0;
      }
      if(comByte == 'N'){// Forward paper for one dot
        motorStep(lineHeight*10, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 0);
        stepsDone = 0;
      }
      if(comByte == 'e'){// Move for one empty data byte
        motorStep(dotWidth * 8, DIR_X, STEP_X, STEP_PAUSE_FAST, 0);
        stepsDone += dotWidth * 8;
      }


      if(comByte == 'T'){// Test plasma print
        for(i = 0; i < 20; i++){
          makeDot(55 + i * 5);
          motorStep(300, DIR_X, STEP_X, STEP_PAUSE_FAST, 0);
        }
        motorStep(1000, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 1);
        motorStep(-300 * 20, DIR_X, STEP_X, STEP_PAUSE_FAST, 1);        
      }

      if(comByte == 'c'){// Cut paper
        for(i = 0; i < 66; i++){
          makeDot(55);
          delay(30);
          motorStep(120, DIR_X, STEP_X, STEP_PAUSE, 0);
        }
        motorStep(2000, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 1);
        motorStep(-120 * 66, DIR_X, STEP_X, STEP_PAUSE_FAST, 1);        
      }

      if(comByte == 'f'){// Fire plasma
        for(i = 0; i < 500; i++){
          makeDot(DOT_PAUSE_SHORT);
          delay(50);
        }
      }

      if(comByte == 't'){// Test fire plasma
        makeDot(DOT_PAUSE_SHORT);
      }

      if(comByte == 'p'){// Paper out
        motorStep(50000, DIR_Y, STEP_Y, STEP_PAUSE_FAST, 1);
      }

      if(comByte == 'h'){// Head to right and back 
        for(i=0; i<3; i++){
          motorStep(6000, DIR_X, STEP_X, STEP_PAUSE, 1);
          motorStep(-6000, DIR_X, STEP_X, STEP_PAUSE_FAST, 1);
        }
      }


      if(comByte == 'x'){
        motorStep(100, DIR_X, STEP_X, STEP_PAUSE, 1);
      }
      if(comByte == 'X'){
        motorStep(-100, DIR_X, STEP_X, STEP_PAUSE, 1);
      }
      if(comByte == 'y'){
        motorStep(100, DIR_Y, STEP_Y, STEP_PAUSE, 1);
      }
      if(comByte == 'Y'){
        motorStep(-100, DIR_Y, STEP_Y, STEP_PAUSE, 1);
      }

      if(comByte == 's'){// Sleep - disable steppers
        motorStep(0, DIR_Y, STEP_Y, STEP_PAUSE, 1);
      }

    }
    else{
      for(int i = 0; i < 8; i++){
        if(comByte & (1<<i)){
          makeDot(DOT_PAUSE_SHORT);
        }
        motorStep(dotWidth, DIR_X, STEP_X, STEP_PAUSE, 0);
        stepsDone+=dotWidth;
      }
      dataMode--;
    }

  }//if (Serial.available() > 0){






}
